@@ -6,4 +6,4 @@ echo Compiling, please wait...
 gcc -DNDEBUG src/gfx/*.c src/*.c -lSDL2 -lm -Wall -Wno-unused-result -Wc++-compat -Wshadow -Winit-self -Wextra -Wunused -Wunreachable-code -Wredundant-decls -Wswitch-default -march=native -mtune=native -O3 -o release/other/pt2-clone
 rm src/gfx/*.o src/*.o &> /dev/null
 
-echo Done! The executable is in the folder named \'release/other\'.
\ No newline at end of file
+echo Done. The executable can be found in \'release/other\' if everything went well.
